/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.commons.events;

import java.util.List;

import org.almendra.commons.util.collection.IListEvaluator;


/**
 * Holds a list of event handlers that should be invoked when an event has
 * occurred.
 * 
 * @author Oliver
 */
public class EventNotifier<T extends IEventHandler>
{
	/**
	 * List of event handlers to be invoked when event occurs.
	 */
	protected IListEvaluator<T> eventHandlers = null;

	/**
	 * Creates a new <code>EventNotifier</code> object. Declared private to
	 * enforce the use of the public constructors to make sure the object is
	 * initialized propperly.
	 */
	protected EventNotifier()
	{
		super();
	}
	
	/**
	 * Creates a new <code>EventNotifier</code> object.
	 */
	public EventNotifier(IListEvaluator<T> eventHandlers)
	{
		super();
		this.eventHandlers = eventHandlers;
	}
	
	/**
	 * Notify all event handlers that the event they have registered for has occurred.
	 * 
	 * @param event
	 */
	public void occured(Event event)
	{
		List<T> handlers = getEventHandlers().get();
		for(T handler : handlers)
			handler.occured(event);
	}
	
	public void addEventHandler(T handler)
	{
		getEventHandlers().get().add(handler);
	}

	/**
	 * @return Returns the eventHandlers.
	 */
	public IListEvaluator<T> getEventHandlers()
	{
		return eventHandlers;
	}

}
