/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.commons.util.inet;

import java.io.Serializable;
import java.rmi.registry.Registry;

import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.StringUtility;


/**
 * @author Oliver
 *
 */

public class RegistryLocation implements Serializable
{
	private static final long serialVersionUID = -1549913713533155856L;
	protected String hostName = null;
	protected int port = Registry.REGISTRY_PORT;
	
	/**
	 * Creates a new <code>RegistryLocation</code> object.
	 */
	private RegistryLocation()
	{
		super();
	}
	
	/**
	 * Creates a new <code>RegistryLocation</code> object.
	 */
	public RegistryLocation(String hostName)
	{
		super();
		AssertUtility.assertTrue(StringUtility.isValid(hostName));
		this.hostName = hostName;
		this.port = Registry.REGISTRY_PORT;
	}
	
	/**
	 * Creates a new <code>RegistryLocation</code> object.
	 */
	public RegistryLocation(int port)
	{
		super();
		AssertUtility.assertTrue(port > 0);
		this.hostName = InetUtility.getHostName();
		this.port = port;
	}
	
	/**
	 * Creates a new <code>RegistryLocation</code> object.
	 */
	public RegistryLocation(String hostName, int port)
	{
		super();
		AssertUtility.assertTrue(StringUtility.isValid(hostName));
		AssertUtility.assertTrue(port > 0);
		this.hostName = hostName;
		this.port = port;
	}

	/**
	 * @return Returns the hostName.
	 */
	public String getHostName()
	{
		return hostName;
	}

	/**
	 * @return Returns the port.
	 */
	public int getPort()
	{
		return port;
	}

}
