package org.almendra.janet.ade.demo.fibonacci;

import java.awt.Frame;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.DefaultListModel;
import javax.swing.JOptionPane;

import org.almendra.commons.events.IEventHandler;
import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.collection.ListUtility;
import org.almendra.commons.util.logging.ErrorLogger;
import org.almendra.commons.util.swing.JTextPaneHolder;
import org.almendra.commons.util.swing.ListModelUtility;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.agents.MultiAgentProxy;
import org.almendra.janet.cas.events.EventHandlerEnvelope;
import org.almendra.janet.cas.events.HandlerList;
import org.almendra.janet.cas.handlers.ForeignApplicationRegisteredParams;
import org.almendra.janet.cas.handlers.NodeDeregisteredParams;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.ApplicationAccessor;
import org.almendra.janet.cas.node.IllegalAgentAccessException;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.node.NoSuchAgentException;
import org.almendra.janet.cas.node.NodeAccessor;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IWorkLoadBalancingCommand;
import org.almendra.janet.cas.scheduling.IWorkLoadSharingCommand;


/**
 * Insert the type's description here. Creation date: (29.03.2004 12:32:32)
 * @author: Administrator
 */
public class FibonacciDemoViewController
{
	public static final int MAX_FIBO_NUMBER = 50000;
	public static final int MAX_TIMES_NUMBER = 100;
	public static final int MAX_COMMANDS_NUMBER = 500;

	protected static final String APPLICATION_AGENT_NAMES_KEY = "FIBONACCI_AGENT_PROXY"; //$NON-NLS-1$
	protected static final String APPLICATION_KEY = "FIBONACCI_APPLICATION"; //$NON-NLS-1$
	protected static final String APPLICATION_LIST_NAME = "FIBONACCI_APPLICATION_LIST"; //$NON-NLS-1$
	protected static final String NODE_NAMES_KEY = "FIBONACCI_APPLICATION_LIST"; //$NON-NLS-1$
	protected static final String LABEL = "AD.Fibonacci Sample"; //$NON-NLS-1$
	
	public static final int CommandTypeSharing = 0;
	public static final int CommandTypeBalancing = 1;
	
	protected FibonacciDemoView view = null;
	protected JTextPaneHolder textPaneHolder = null;
	protected Map<String, ApplicationInfo> allApplicationObjects = null;
	protected Set<String> nodeNames = null;
	protected String nodeName = null;
	protected Logger logger = null;
	protected NodeAccessor node = null;
	
	/**
	 * FibonacciDemoViewController constructor comment.
	 */
	public FibonacciDemoViewController(FibonacciDemoView view, Map<String, ApplicationInfo> allApplicationObjects, Set<String> nodeNames, CommandAccessor cmdAccessor)
	{
		super();
		this.view = view;
		this.allApplicationObjects = allApplicationObjects;
		this.nodeNames = nodeNames;
		this.nodeName = cmdAccessor.getNodeName();
		this.logger = cmdAccessor.getLogger();
		this.node = cmdAccessor.getNodeAccessor();
		registerForEvents(cmdAccessor);
	}

	/**
	 * @param registry
	 */
	protected void registerForEvents(CommandAccessor cmdAccessor)
	{
		IEventHandler handler = new ForeignApplicationRegisteredHandler(this);
		EventHandlerEnvelope env = new EventHandlerEnvelope(handler, cmdAccessor.getCurrentAgent().getAgentPath());
		HandlerList handlerList = new HandlerList(env);
		cmdAccessor.getNodeEventRegistry().register(ForeignApplicationRegisteredParams.EVENT_NAME, handlerList, cmdAccessor.getApplication());
		
		handler = new NodeDeregisteredHandler(this);
		env = new EventHandlerEnvelope(handler, cmdAccessor.getCurrentAgent().getAgentPath());
		handlerList = new HandlerList(env);
		cmdAccessor.getNodeEventRegistry().register(NodeDeregisteredParams.EVENT_NAME, handlerList, cmdAccessor.getApplication());
	}

	/**
	 * Insert the method's description here. Creation date: (29.03.2004
	 * 12:40:43)
	 */
	protected void shutdown()
	{
		getTextPaneHolder().clear();
	}

	/**
	 * 
	 */
	public void close()
	{
		view.dispose();
	}
	
	/**
	 * @return
	 */
	public JTextPaneHolder getTextPaneHolder()
	{
		if(textPaneHolder == null)
		{
			textPaneHolder = new JTextPaneHolder(view.getJTextPane()); 
		}
		return textPaneHolder;
	}

	/**
	 * 
	 * @param frame
	 * @param nthFiboNumberString
	 * @param timesString
	 */
	public void start(Frame frame, List<String> nodeNamesOrNull, String agentName, String appName, String nthFiboNumberString, String timesString, String timesCommandString, int commandType)
	{
		Integer nthFibonacciNumberInt = StringUtility.toInteger(nthFiboNumberString);
		if(nthFibonacciNumberInt == null)
		{
			JOptionPane.showMessageDialog(frame, "Invalid number: " + nthFiboNumberString); //$NON-NLS-1$
			return;
		}
		
		int nthFibonacciNumber = nthFibonacciNumberInt.intValue();
		if(nthFibonacciNumber > MAX_FIBO_NUMBER)
		{
			JOptionPane.showMessageDialog(frame, "Input value too large for fibonacci value (max: " + MAX_FIBO_NUMBER + ")!"); //$NON-NLS-1$ //$NON-NLS-2$
			return;
		}
		
		Integer timesInt = StringUtility.toInteger(timesString);
		if(timesInt == null)
		{
			JOptionPane.showMessageDialog(frame, "Invalid number: " + timesString); //$NON-NLS-1$
			return;
		}
		
		int times = timesInt.intValue();
		if(times > MAX_TIMES_NUMBER)
		{
			JOptionPane.showMessageDialog(frame, "Input value too large for repeat value (max: " + MAX_TIMES_NUMBER + ")!"); //$NON-NLS-1$ //$NON-NLS-2$
			return;
		}
		
		Integer timesCommandInt = StringUtility.toInteger(timesCommandString);
		if(timesCommandInt == null)
		{
			JOptionPane.showMessageDialog(frame, "Invalid number: " + timesCommandString); //$NON-NLS-1$
			return;
		}
		
		int timesCommands = timesCommandInt.intValue();
		if(timesCommands > MAX_COMMANDS_NUMBER)
		{
			JOptionPane.showMessageDialog(frame, "Input value too large for number of commands value (max: " + MAX_COMMANDS_NUMBER + ")!"); //$NON-NLS-1$ //$NON-NLS-2$
			return;
		}
		
		start(nodeNamesOrNull, agentName, appName, nthFibonacciNumber, times, timesCommands, commandType);
	}

	/**
	 * @param nthFibonacciNumber
	 * @param i
	 */
	protected void start(List<String> nodeNamesOrNull, String agentName, String appName, int nthFibonacciNumber, int times, int timesCommands, int commandType)
	{
		getTextPaneHolder().clearText();
		IAgentProxy fibonacciAgent = getFibonacciAgent(nodeNamesOrNull, agentName, appName);
		if(fibonacciAgent == null)
			return;
		
		try
		{
			if(commandType == CommandTypeBalancing)
			{
				for (int i = 0; i < timesCommands; i++)
				{	
					IWorkLoadBalancingCommand balancingCmd = new FibonacciBalancingCommand(nthFibonacciNumber, times);
					fibonacciAgent.accept(balancingCmd);
				}
				return;
			}
			if(commandType == CommandTypeSharing)
			{
				for (int i = 0; i < timesCommands; i++)
				{
					IWorkLoadSharingCommand sharingCmd = new FibonacciSharingCommand(nthFibonacciNumber, times);
					fibonacciAgent.accept(sharingCmd);
				}
				return;
			}
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			ErrorLogger.exception(e);
		}		
	}

	/**
	 * @param appName
	 * @return
	 */
	protected IAgentProxy getFibonacciAgent(List<String> nodeNamesOrNull, String agentName, String appName)
	{
		String agentNodeName = null;
		if(nodeNamesOrNull == null)
			agentNodeName = nodeName; 
		if(nodeNamesOrNull != null && nodeNamesOrNull.size() == 1)
			agentNodeName = nodeNamesOrNull.get(0);
		
		if(agentNodeName != null)
		{
			AgentPath path = new AgentPath(agentNodeName, appName, FibonacciUtility.CAP_NAME, agentName);
			ApplicationInfo appInfo = allApplicationObjects.get(appName);
			ApplicationAccessor app = appInfo.getApp();
						
			try
			{
				return app.getAgent(path);
			}
			catch (NoSuchAgentException e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				logger.exception(e);
			}
			catch (IllegalAgentAccessException e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				logger.exception(e);
			}
			return null;
		}
		
		if(nodeNamesOrNull.isEmpty())
			JOptionPane.showMessageDialog(view, "No node name specified!"); //$NON-NLS-1$
		
		List<IAgentProxy> agentProxies = new ArrayList<IAgentProxy>();
		for(String currentNodeName : nodeNamesOrNull)
		{
			try
			{
				AgentPath agentPath = new AgentPath(currentNodeName, appName, FibonacciUtility.CAP_NAME, agentName);
				ApplicationInfo appInfo = allApplicationObjects.get(appName);
				ApplicationAccessor app = appInfo.getApp();
				agentProxies.add(app.getAgent(agentPath));
			}
			catch (NoSuchAgentException e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				logger.exception(e);
			}
			catch (IllegalAgentAccessException e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				logger.exception(e);
			}
		}
		if(agentProxies.isEmpty())
			return null;
		return new MultiAgentProxy(agentProxies);
	}

	/**
	 * @return
	 */
	public String getNodeName()
	{
	    for(ApplicationInfo appInfo : allApplicationObjects.values())
			return appInfo.getNodeName();
		return "?"; //$NON-NLS-1$
	}

	/**
	 * @return
	 */
	public List<String> getAppNamesSorted()
	{
		List<String> appNames = ListUtility.toStringList(allApplicationObjects.keySet());
		Collections.sort(appNames);
		return appNames;
	}

	/**
	 * @return
	 */
	public List<String> getNodeNamesSorted()
	{
		rereadNodeNames();
		List<String> nodeNamesList = ListUtility.toStringList(nodeNames);
		Collections.sort(nodeNamesList);
		return nodeNamesList;
	}

	/**
	 * 
	 */
	protected void rereadNodeNames()
	{
		List<String> allNodeNames = node.getAllNodeNames();
		allNodeNames.removeAll(nodeNames);
		if(allNodeNames.isEmpty())
			return;
		for(String nodeName : allNodeNames)
		{
			if(node.hasApplication(nodeName, FibonacciUtility.APP_NAME_1))
				nodeNames.add(nodeName);
		}
	}

	/**
	 * @param string
	 */
	public void deregisteredNode(String nodeName)
	{
		if(!nodeNames.contains(nodeName))
			return;
		nodeNames.remove(nodeName);
		DefaultListModel model = (DefaultListModel) view.getJListNodeNames().getModel();
		Object selectedValue = view.getJListNodeNames().getSelectedValue();
		model.removeElement(nodeName);
		if(selectedValue != null && !selectedValue.equals(nodeName))
			view.getJListNodeNames().setSelectedValue(selectedValue, true);
	}

	/**
	 * @param string
	 */
	public void registeredNode(String nodeName)
	{
		nodeNames.add(nodeName);
		DefaultListModel model = (DefaultListModel) view.getJListNodeNames().getModel();
		Object selectedValue = view.getJListNodeNames().getSelectedValue();
		model.clear();
		for(String name : getNodeNamesSorted())
			model.addElement(name);
		if(selectedValue != null)
			view.getJListNodeNames().setSelectedValue(selectedValue, true);
		
	}

	/**
	 * 
	 */
	public void selectedApplicationChanged()
	{
		String appName = (String) view.getJApplicationComboBox().getSelectedItem();
		if(appName == null)
			return;
		ApplicationInfo appInfo = allApplicationObjects.get(appName);
		String currentAgentName = (String) view.getJComboBoxAgents().getSelectedItem();
		view.getJComboBoxAgents().removeAllItems();
		for(String agentName : appInfo.getAgentNames())
			view.getJComboBoxAgents().addItem(agentName);
		if(currentAgentName != null && appInfo.getAgentNames().contains(currentAgentName))
			view.getJComboBoxAgents().setSelectedItem(currentAgentName);
		else
			view.getJComboBoxAgents().setSelectedIndex(0);
	}

	/**
	 * 
	 */
	public void setDefaults()
	{
		view.setTitle(view.getTitle() + " on " + nodeName); //$NON-NLS-1$
		DefaultListModel model = (DefaultListModel) view.getJListNodeNames().getModel();
		for(String name : getNodeNamesSorted())
			model.addElement(name);
		view.getJListNodeNames().setSelectedValue(nodeName, true);
		for(String appName : getAppNamesSorted())
			view.getJApplicationComboBox().addItem(appName);
		view.getJApplicationComboBox().setSelectedIndex(0);
	}

	/**
	 * @param vector
	 */
	public void registeredNode(List<String> nodeNames)
	{
		DefaultListModel model = (DefaultListModel) view.getJListNodeNames().getModel();
		for(String nodeName : nodeNames)
		{
			if(!ListModelUtility.contains(model, nodeName))
				registeredNode(nodeName);
		}
	}

}
