package org.almendra.janet.ade.observer.simulation.gui;

import java.io.File;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.swing.DefaultListModel;
import javax.swing.JOptionPane;

import org.almendra.commons.util.collection.ListUtility;
import org.almendra.commons.util.collection.StringList;
import org.almendra.commons.util.swing.TextAreaPanelController;
import org.almendra.janet.ade.observer.ObserverAnchor;
import org.almendra.janet.ade.observer.gui.IDisplayController;
import org.almendra.janet.ade.observer.gui.ObserverViewController;
import org.almendra.janet.ade.observer.simulation.ObserverAnchorSimulation;
import org.almendra.janet.cas.node.Node;


/**
 * Insert the type's description here. Creation date: (05.04.2004 12:42:41)
 * @author: Administrator
 */
public class ObserverSimulationViewController extends ObserverViewController implements IDisplayController
{
	protected ObserverSimulationView view = null;
	protected ObserverAnchorSimulation anchor = null;
	protected Node node = null; 
	protected List<String> selectedNodes = new StringList();
	protected TextAreaPanelController textArea = null;

	/**
	 * ObserverSimulationViewController constructor comment.
	 */
	public ObserverSimulationViewController()
	{
		super();
	}

	/**
	 * ObserverSimulationViewController constructor comment.
	 */
	public ObserverSimulationViewController(ObserverSimulationView view)
	{
		super();
		this.view = view;
		textArea = new TextAreaPanelController(view.getJProcessesTextArea(), 20);
	}

	public void newNonJanetCPULoad(int value)
	{
		anchor.setSimulatedNonJanetCPULoad(value);
	}

	public void searchNodes()
	{
		setNodesToObserve(null);
	}

	public void setDefaults()
	{
		setNodesToObserve(null);
		textArea.setHeader(ListUtility.toStringListFromSingleObject(ObserverAnchor.ALL_OTHERS_LABEL), ObserverViewController.TextAreaIndentation);
	}

	/**
	 * 
	 */
	protected void setNodesToObserve(String directoryPathNameOrNull)
	{
		DefaultListModel listModel =  (DefaultListModel) view.getJNodesList().getModel();
		listModel.removeAllElements();
		
		File file = null;
		if(directoryPathNameOrNull == null)
			file = new File("."); //$NON-NLS-1$
		else
			file = new File(directoryPathNameOrNull); 
		String[] fileNames = file.list(new ExecutorLoadFilenameFilter());
		if(fileNames.length == 0)
			return;
		List<String> fileNamesList = new StringList(fileNames);		
		List<String> nodeNamesList = new StringList();
		
		for(String fileName : fileNamesList)
		{
			String nodeName = fileName.substring(0, fileName.lastIndexOf("-")); //$NON-NLS-1$
			nodeNamesList.add(nodeName);
		}
		
		Collections.sort(nodeNamesList);		
		for (int i = 0; i < nodeNamesList.size(); i++)
			listModel.addElement(nodeNamesList.get(i));

	}

	public void shutdown()
	{
		node.shutdown();
	}
	
	/**
	 * @return Returns the anchor.
	 */
	public ObserverAnchorSimulation getAnchor()
	{
		return anchor;
	}
	/**
	 * @param anchor The anchor to set.
	 */
	public void setAnchor(ObserverAnchorSimulation anchor)
	{
		this.anchor = anchor;
		anchor.setDisplayController(this);
	}
	/**
	 * @return Returns the node.
	 */
	public Node getNode()
	{
		return node;
	}
	/**
	 * @param node The node to set.
	 */
	public void setNode(Node node)
	{
		this.node = node;
		view.setTitle(view.getTitle() + " on " + node.getName()); //$NON-NLS-1$
	}

	/**
	 * 
	 */
	public void nodesListChanged(Object[] selectedObjects)
	{
		if(!selectionChanged(selectedObjects))
			return;
		
		if(!notifyAnchorNodesListChanged(selectedObjects))
		{
			searchNodes();	// same as remove selection
			return;
		}
		
		for(Object selectedObject : selectedObjects)
			selectedNodes.add((String) selectedObject);

		anchor.setNodeNames(selectedNodes);
		resetDisplayNodes();
		
	}

	/**
	 * @param selectedObjects
	 */
	protected boolean notifyAnchorNodesListChanged(Object[] selectedObjects)
	{
		List<String> selectedNodeNames = new StringList();
		for(Object selectedObject : selectedObjects)
			selectedNodeNames.add((String) selectedObject);
		
		if(selectedNodeNames.size() > selectedNodes.size())
		{
			for(String nodeName : selectedNodeNames)
			{
				if(!selectedNodes.contains(nodeName))
				{
					String error = anchor.notifyNodeAdded(nodeName);
					if(error.length() > 0)
					{
						JOptionPane.showMessageDialog(view, error);
						return false;
					}
				}
			}
			return true;
		}
		
		
		if(selectedNodeNames.size() < selectedNodes.size())
		{
			for(String nodeName : selectedNodes)
			{
				if(!selectedNodeNames.contains(nodeName))
				{
					String error = anchor.notifyNodeRemoved(nodeName);
					if(error.length() > 0)
					{
						JOptionPane.showMessageDialog(view, error);
						return false;
					}
				}
			}
			return true;
		}
		
		return true;		
	}

	/**
	 * @param selectedNodeNames
	 */
	protected void resetDisplayNodes()
	{
		textArea.setHeader(getHeaderValues(), ObserverViewController.TextAreaIndentation);
	}
	
	protected List<String> getHeaderValues()
	{
		List<String> headerValues = new StringList(selectedNodes);
		headerValues.add(0, ObserverAnchor.ALL_OTHERS_LABEL);
		return headerValues;
	}

	/**
	 * @param selectedObjects
	 * @return
	 */
	protected boolean selectionChanged(Object[] selectedObjects)
	{
		if(selectedNodes.size() != selectedObjects.length)
			return true;
		for (int i = 0; i < selectedObjects.length; i++)
		{
			if(!selectedNodes.contains(selectedObjects[i]))
				return true;
		}
		return false;
	}
	/**
	 * @return Returns the selectedNodes.
	 */
	public List<String> getSelectedNodes()
	{
		return selectedNodes;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.observer.gui.IDisplayController#displayValues(java.util.Map)
	 */
	public void displayValues(Map<String, String> values)
	{
		textArea.insertBelowHeader(getValuesInOrder(values, getHeaderValues()));
	}
	
	public List<String> getValuesInOrder(Map<String, String> map, List<String> keysInOrder)
	{
		List<String> values = new StringList();
		for(String key : keysInOrder)
		{
			String value = map.get(key);
			if(value == null)
				value = "-"; //$NON-NLS-1$
			values.add(value);
		}
		return values;
	}
}
