/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.ade.commands;

import java.io.Serializable;
import java.util.List;

import org.almendra.commons.util.collection.StringList;
import org.almendra.janet.cas.scheduling.ICommand;


/**
 * @author Oliver Plohmann
 *
 */
public class ProcessQSCChangedCommand implements ICommand, Serializable
{
	private static final long serialVersionUID = 4282681135208780146L;

	public static final String QualifiedName = "org.almendra.commons.janet.ade.commands.ProcessQSCChangedCommand"; //$NON-NLS-1$
	
	public static final int NodeDroppedToQSC0 = 1;
	public static final int NodesWithQSC0Exist = 2;
	
	protected String nodeName = null;
	protected int change = -1;
	protected List<String> nodeNames = new StringList();
	
	/**
	 * Creates a new <code>ProcessQSCChangedCommand</code> object.
	 */
	protected ProcessQSCChangedCommand()
	{
		super();
	}
	
	/**
	 * Creates a new <code>ProcessQSCChangedCommand</code> object.
	 */
	public ProcessQSCChangedCommand(int change)
	{
		super();
		this.change = change;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getQualifiedName()
	 */
	public String getQualifiedName()
	{
		return QualifiedName;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#getPriority()
	 */
	public int getPriority()
	{
		return ICommand.ApplicationMaxPriority;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.ICommand#toLogString()
	 */
	public String toLogString()
	{
		return "ProcessQSCChangedCommand"; //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() throws CloneNotSupportedException
	{
		ProcessQSCChangedCommand cmd = (ProcessQSCChangedCommand) super.clone();
		return cmd;
	}
	/**
	 * @return Returns the change.
	 */
	public int getChange()
	{
		return change;
	}
	/**
	 * @param change The change to set.
	 */
	public void setChange(int change)
	{
		this.change = change;
	}
	/**
	 * @return Returns the nodeName.
	 */
	public String getNodeName()
	{
		return nodeName;
	}
	/**
	 * @param nodeName The nodeName to set.
	 */
	public void setNodeName(String nodeName)
	{
		this.nodeName = nodeName;
	}
	/**
	 * @return Returns the nodeNames.
	 */
	public List<String> getNodeNames()
	{
		return nodeNames;
	}
	/**
	 * @param nodeNames The nodeNames to set.
	 */
	public void setNodeNames(List<String> nodeNames)
	{
		this.nodeNames = nodeNames;
	}
}
