/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.oliverplohmann.de/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.ade.distributor;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.almendra.commons.number.Long;
import org.almendra.commons.util.collection.StringList;



/**
 * @author Oliver Plohmann
 *
 */
public class CapabilityQueueSizeStore
{
	protected Map<String, Set<String>> nodesByCapability = new HashMap<String, Set<String>>();	//	<CapabilityPathAsString, Set<NodeNameAsString>>
	protected Map<String, Set<String>> capabilitiesByNode = new HashMap<String, Set<String>>();	//	<NodeName, Set<CapabilityPathAsString>>
	protected Map<String, Long> queueSizeByCapabilityAndNode = new HashMap<String, Long>();
	
	/**
	 * 
	 */
	public CapabilityQueueSizeStore()
	{
		super();
	}
	
	protected void removeCapabilitiesNotInUse(Map<String, Long> queuesByCapability, String nodeName)
	{
		List<String> unusedCapabilities = new StringList();
		Set<String> currentCapabilities = capabilitiesByNode.get(nodeName);
		if(currentCapabilities != null)
		{			
			for(String capabilityPath : currentCapabilities)
			{
				if(!queuesByCapability.containsKey(capabilityPath))
					unusedCapabilities.add(capabilityPath);
			}
		}
		
		for(String unusedCapability : unusedCapabilities)
		{
			Set<String> nodes = nodesByCapability.get(unusedCapability);
			if(nodes != null)
				nodes.remove(nodeName);
			Set<String> capabilities = capabilitiesByNode.get(nodeName);
			if(capabilities != null)
				capabilities.remove(unusedCapability);
			queueSizeByCapabilityAndNode.remove(unusedCapability + "." + nodeName); //$NON-NLS-1$
		}
	}
	
	public void update(Map<String, Long> queuesByCapability, String nodeName)
	{
		removeCapabilitiesNotInUse(queuesByCapability, nodeName);
		insertCapabilitiesInUse(queuesByCapability, nodeName);		
	}
	
	public void removeEntries(String nodeName)
	{
		Set<String> capabilities = capabilitiesByNode.remove(nodeName);
		if(capabilities == null || capabilities.isEmpty())
			return;	
		for(String capabilityPath : capabilities)
		{
			Set<String> nodeNames = nodesByCapability.get(capabilityPath);
			if(nodeNames != null)
				nodeNames.remove(nodeName);
			queueSizeByCapabilityAndNode.remove(capabilityPath + "." + nodeNames); //$NON-NLS-1$
		}
	}
	
	protected void insertCapabilitiesInUse(Map<String, Long> queuesByCapability, String nodeName)
	{
		for(String capabilityPath : queuesByCapability.keySet())
		{
		    Long queueSize = queuesByCapability.get(capabilityPath);
			Set<String> nodesNames = nodesByCapability.get(capabilityPath);
			if(nodesNames == null)
			{
				nodesNames = new HashSet<String>();
				nodesByCapability.put(capabilityPath, nodesNames);
			}
			nodesNames.add(nodeName);
			Set<String> capabilities = capabilitiesByNode.get(nodeName);
			if(capabilities == null)
			{
				capabilities = new HashSet<String>();
				capabilitiesByNode.put(nodeName, capabilities);
			}
			capabilities.add(capabilityPath);
			queueSizeByCapabilityAndNode.put(capabilityPath + "." + nodeName, queueSize); //$NON-NLS-1$
		}	
	}
	
	public Set getCapabilities(String nodeName)
	{
		Set<String> capabilities = capabilitiesByNode.get(nodeName);
		if(capabilities == null)
			capabilities = new HashSet<String>();
		return capabilities;
	}
	
	public Set getNodes(String capabilityPath)
	{
		Set<String> nodes = nodesByCapability.get(capabilityPath);
		if(nodes == null)
			nodes = new HashSet<String>();
		return nodes;
	}

	/**
	 * @param nodeName
	 * @param capabilityPath
	 */
	public int getQueueSize(String nodeName, String capabilityPath)
	{
		Long queueSize = queueSizeByCapabilityAndNode.get(capabilityPath + "." + nodeName); //$NON-NLS-1$
		if(queueSize == null)
			return 0;
		return (int) queueSize.longValue();
	}
}
