/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.distributor;

import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.xml.XMLParseException;
import org.jdom.Element;



public class Descriptor
{
	public static final String TAG_SHARING_DISTRIBUTOR = "sharingDistributor"; //$NON-NLS-1$
	public static final String TAG_SHARING_DISTRIBUTOR_DELAY = "delayLastConsideredNode"; //$NON-NLS-1$
	public static final String TAG_BALANCING = "balancing";  //$NON-NLS-1$
	public static final String TAG_DISABLE_EVICTION = "disableEviction";  //$NON-NLS-1$
	public static final String TAG_EVICT_COMMANDS_IF_CPU_LOAD_ABOVE = "evictCommandsIfNonJanetCPULoadAbovePercent";  //$NON-NLS-1$
	
	protected volatile long sharingDelay = -1;
	protected volatile boolean disableEviction = false;
	protected volatile boolean disableFullEviction = false;
	protected volatile boolean disablePartialEviction = false;
	protected volatile boolean disableNodesWithQSC0Exist = false;
	protected volatile boolean disableNodeDroppedToQSC1 = false;
	
	/**
	 * Creates a new <code>Descriptor</code> object.
	 */
	public Descriptor()
	{
		super();
	}

	/**
	 * @param root
	 * @return
	 */
	public static Descriptor create(Element root) throws XMLParseException
	{
		Descriptor desc = new Descriptor();
		desc.fill(root);
		return desc;
	}

	/**
	 * @param root
	 */
	protected void fill(Element root)
	{				
		fillDistributorSharing(root);		
		fillDistributorBalancing(root);	
	}
	
	protected void fillDistributorBalancing(Element root)
	{
		Element balancingElement = root.getChild(TAG_BALANCING);
		if (balancingElement == null)
			return;
		
		String evictCommandsCPULoadString = balancingElement.getAttributeValue(TAG_EVICT_COMMANDS_IF_CPU_LOAD_ABOVE);
		if(evictCommandsCPULoadString == null)
			return;
		
		String disableEvictionString = balancingElement.getAttributeValue(TAG_DISABLE_EVICTION);
		if(disableEvictionString != null)
		{
			disableEvictionString = disableEvictionString.toLowerCase();
			if(disableEvictionString.equals("true")) //$NON-NLS-1$
				disableEviction = true;
			if(disableEvictionString.equals("false")) //$NON-NLS-1$
				disableEviction = false;
		}	
	}
	
	protected void fillDistributorSharing(Element root)
	{
		Element distributorSharingElement = root.getChild(TAG_SHARING_DISTRIBUTOR);
		if (distributorSharingElement == null)
			return;
		
		String sharingDelayString = distributorSharingElement.getAttributeValue(TAG_SHARING_DISTRIBUTOR_DELAY);
		if(sharingDelayString == null)
			return;
		
		Integer delayInt = StringUtility.toInteger(sharingDelayString);
		if(delayInt == null)
			return;
				
		sharingDelay = delayInt.longValue();
	}

	/**
	 * @return
	 */
	public long getSharingDelay()
	{
		return sharingDelay;
	}

	/**
	 * @return Returns the disableEviction.
	 */
	public boolean isDisableEviction()
	{
		return disableEviction;
	}
	/**
	 * @return Returns the disableFullEviction.
	 */
	public boolean isDisableFullEviction()
	{
		return disableFullEviction;
	}
	/**
	 * @param disableFullEviction The disableFullEviction to set.
	 */
	public void setDisableFullEviction(boolean disableFullEviction)
	{
		this.disableFullEviction = disableFullEviction;
	}
	/**
	 * @return Returns the disableNodeDroppedToQSC1.
	 */
	public boolean isDisableNodeDroppedToQSC1()
	{
		return disableNodeDroppedToQSC1;
	}
	/**
	 * @param disableNodeDroppedToQSC1 The disableNodeDroppedToQSC1 to set.
	 */
	public void setDisableNodeDroppedToQSC1(boolean disableNodeDroppedToQSC1)
	{
		this.disableNodeDroppedToQSC1 = disableNodeDroppedToQSC1;
	}
	/**
	 * @return Returns the disableNodesWithQSC0Exist.
	 */
	public boolean isDisableNodesWithQSC0Exist()
	{
		return disableNodesWithQSC0Exist;
	}
	/**
	 * @param disableNodesWithQSC0Exist The disableNodesWithQSC0Exist to set.
	 */
	public void setDisableNodesWithQSC0Exist(boolean disableNodesWithQSC0Exist)
	{
		this.disableNodesWithQSC0Exist = disableNodesWithQSC0Exist;
	}
	/**
	 * @return Returns the disablePartialEviction.
	 */
	public boolean isDisablePartialEviction()
	{
		return disablePartialEviction;
	}
	/**
	 * @param disablePartialEviction The disablePartialEviction to set.
	 */
	public void setDisablePartialEviction(boolean disablePartialEviction)
	{
		this.disablePartialEviction = disablePartialEviction;
	}
	/**
	 * @param disableEviction The disableEviction to set.
	 */
	public void setDisableEviction(boolean disableEviction)
	{
		this.disableEviction = disableEviction;
	}
}
