/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.ade.distributor.interpreters;

import java.util.List;

import org.almendra.janet.ade.commands.ProcessQSCChangedCommand;
import org.almendra.janet.ade.distributor.DistributorAnchor;
import org.almendra.janet.cas.scheduling.CommandAccessor;


/**
 * @author Oliver Plohmann
 *
 */
public abstract class NotifyValueChangedInterpreter extends DistributorInterpreter
{
	/**
	 * Creates a new <code>NotifyValueChangedInterpreter</code> object.
	 */
	public NotifyValueChangedInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#execute(org.almendra.commons.janet.cas.scheduling.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public List<String> commandNames()
	{
		return null;
	}

	/**
	 * @param nodeName
	 * @param anchor
	 * @throws RemoteException
	 * @throws InvalidCommandException
	 */
	protected void reflectNodeDroppedToQSC0(String nodeName, CommandAccessor cmdAccessor, DistributorAnchor anchor)
	{
		try
		{			 
			if(anchor.getNodeLoadImage(nodeName).isAboveThreshold())
				return;
			List<String> nodeNamesOnQSC2OrAbove = anchor.getAvailableNodeNamesAboveQSC(1);
			nodeNamesOnQSC2OrAbove.remove(nodeName);
			if(nodeNamesOnQSC2OrAbove.isEmpty())
				return;
			ProcessQSCChangedCommand command = new ProcessQSCChangedCommand(ProcessQSCChangedCommand.NodeDroppedToQSC0);
			command.setNodeName(nodeName);
			command.setNodeNames(nodeNamesOnQSC2OrAbove);
			anchor.getBalancerAgent().accept(command);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.getLogger().exception(e);
		}
		return;
	}
}
