/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.ade.executor.executors;

import java.util.List;

import org.almendra.commons.threads.IExecutable;
import org.almendra.janet.ade.executor.ExecutorAnchor;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.node.SystemApplication;
import org.almendra.janet.cas.scheduling.ICommandEnvelope;



public abstract class EvictCommandsExecutor implements IExecutable
{
	protected String capabilityPath = null;
	protected AgentPath evictionAgentPath = null;
	protected IAgentProxy evictionAgent = null;
	protected SystemApplication systemApplication = null;
	protected boolean isInterrupted = false;
	protected ExecutorAnchor anchor = null;
	protected String token = null;
	protected int evictedCommandsCount = 0;
	
	/**
	 * Creates a new <code>EvictCommandsExecutor</code> object.
	 */
	protected EvictCommandsExecutor()
	{
		super();
	}
	
	/**
	 * Creates a new <code>EvictCommandsExecutor</code> object.
	 */
	public EvictCommandsExecutor(SystemApplication systemApplication, AgentPath evictionAgentPath, String capabilityPath, String token)
	{
		super();
		this.systemApplication = systemApplication;
		this.capabilityPath = capabilityPath;
		this.evictionAgentPath = evictionAgentPath;
		this.token = token;
	}
	
	/**
	 * Creates a new <code>EvictCommandsExecutor</code> object.
	 */
	public EvictCommandsExecutor(String token)
	{
		super();
		this.token = token;
	}

	public IAgentProxy getEvictionAgent()
	{			
		try
		{
			return systemApplication.getCrossAgent(evictionAgentPath);
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			getLogger().exception(e);
			// TODO - log error message
		}
		return null;
	}

	/**
	 * @return
	 */
	protected Logger getLogger()
	{
		return systemApplication.getLogger();
	}
	
	protected Node getNode()
	{
		return systemApplication.getNode();
	}
	/**
	 * @return Returns the isInterrupted.
	 */
	public boolean isInterrupted()
	{
		return isInterrupted;
	}
	/**
	 * @param isInterrupted The isInterrupted to set.
	 */
	public void setInterrupted(boolean isInterrupted)
	{
		this.isInterrupted = isInterrupted;
	}
	
	protected boolean checkIsInterrupted()
	{
		if(!anchor.existsToken(token))
			isInterrupted = true;
		return isInterrupted;
	}

	/**
	 * @return Returns the evictedCommandsCount.
	 */
	public int getEvictedCommandsCount()
	{
		return evictedCommandsCount;
	}
	
	public int incrementEvictionCommandsCount()
	{
		evictedCommandsCount++;
		return evictedCommandsCount;
	}

	protected ICommandEnvelope getNextCommand(List<ICommandEnvelope> commands)
	{		
		if(commands == null || commands.isEmpty())
			return null;
		for (int i = commands.size() - 1; i >= 0; i--)
		{
			ICommandEnvelope env = commands.get(i);
			if(env.testSetSuspended())
			{
				setSuspended(env);
				return env;
			}
		}
		return null;
	}

	/**
	 * @param env
	 */
	protected abstract void setSuspended(ICommandEnvelope env);
}
