/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.ade.executor.executors;

import java.util.List;

import org.almendra.janet.ade.executor.ExecutorAnchor;
import org.almendra.janet.ade.executor.handlers.SuspensionHandlerFullEviction;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.handlers.ISuspensionHandler;
import org.almendra.janet.cas.scheduling.ICommandEnvelope;


/**
 * @author Oliver Plohmann
 *
 */
public class EvictExecuteCommandsFullyExecutor extends EvictExecuteCommandsExecutor
{
	List fullEvictionAssignments = null;
	
	/**
	 * Creates a new <code>EvictExecuteCommandsFullyExecutor</code> object.
	 */
	protected EvictExecuteCommandsFullyExecutor()
	{
		super();
	}

	/**
	 * Creates a new <code>EvictExecuteCommandsFullyExecutor</code> object.
	 */
	public EvictExecuteCommandsFullyExecutor(String token, List fullEvictionAssignments)
	{
		super(token);
		this.fullEvictionAssignments = fullEvictionAssignments;
	}
	
	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.threads.IExecutable#execute(java.lang.Object)
	 */
	public void execute(Object object)
	{
		anchor = (ExecutorAnchor) object;
		evictCommandsFully();
	}
	
	protected ISuspensionHandler getSuspensionHandler(ICommandEnvelope env)
	{
		return new SuspensionHandlerFullEviction(anchor, evictionAgent, evictionAgentPath, env, this, capabilityPath);
	}

	/**
	 * 
	 */
	protected void evictCommandsFully()
	{
		if(checkIsInterrupted())
			return;
		int size = anchor.getExecutingCommandsByCapability().keySet().size();
		for (int i = 0; i < size; i++) 
		{
			while(evict1CommandFromCapability())
			{
				if(isInterrupted())
					return;
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.executor.executors.EvictCommandsExecutor#getEvictionAgent()
	 */
	public IAgentProxy getEvictionAgent()
	{
		setEvictionAgent();
		return evictionAgent;
	}
	
	/**
	 * 
	 */
	protected void setEvictionAgent()
	{
		evictionAgent = anchor.getNextLoadSharingAgent();
		evictionAgentPath = anchor.getLoadAdministratorAgentPath(evictionAgent);
	}

}
