/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.ade.executor.executors;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.almendra.commons.threads.IExecutable;
import org.almendra.janet.ade.commands.ExecuteWorkloadCommand;
import org.almendra.janet.ade.commands.WorkloadDistributionCommand;
import org.almendra.janet.ade.executor.ExecutorAnchor;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.scheduling.ICommandEnvelope;


/**
 * @author Oliver Plohmann
 *
 */
public class EvictWaitCommandsFullyExecutor extends EvictWaitCommandsExecutor implements IExecutable
{
	protected List fullEvictionAssignments = null;
	
	/**
	 * Creates a new <code>EvictWaitCommandsFullyExecutor</code> object.
	 */
	public EvictWaitCommandsFullyExecutor()
	{
		super();
	}

	/**
	 * Creates a new <code>EvictWaitCommandsFullyExecutor</code> object.
	 */
	public EvictWaitCommandsFullyExecutor(String token, List fullEvictionAssignments)
	{
		super(token);
		this.fullEvictionAssignments = fullEvictionAssignments;
	}
	
	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.threads.IExecutable#execute(java.lang.Object)
	 */
	public void execute(Object object)
	{
		anchor = (ExecutorAnchor) object;
		evictCommands();
	}
	
	protected void evictCommands()
	{
		if(checkIsInterrupted())
			return;
		Set<String> capabilityPaths = anchor.getWaitingCommandsByCapability().keySet();
		Iterator<String> it = capabilityPaths.iterator();
		while (it.hasNext())
		{
			capabilityPath = it.next();
			while(evict1CommandFromCapability())
			{
				if(isInterrupted())
					return;
			}
		}
	}
	
	/**
	 * @param env
	 */
	protected void prepareEnvelopeBeforeSending(ICommandEnvelope env)
	{
		ExecuteWorkloadCommand workloadCommand = (ExecuteWorkloadCommand) env.getCommand();
		workloadCommand.prepareBeforeSending();
		env.setPredefinedInterpreter(null);
		env.setCommand(new WorkloadDistributionCommand(env.getAgentPath(), workloadCommand));
	}
	
	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.ade.executor.executors.EvictCommandsExecutor#getEvictionAgent()
	 */
	public IAgentProxy getEvictionAgent()
	{
		if(evictionAgent == null)			
		{
			setEvictionAgent();
		}
		return evictionAgent;
	}
	
	/**
	 * 
	 */
	protected void setEvictionAgent()
	{
		evictionAgent = anchor.getNextLoadSharingAgent();
		evictionAgentPath = anchor.getLoadAdministratorAgentPath(evictionAgent);
	}
}
