package org.almendra.janet.cas.demo.fibonacci;

import java.math.BigInteger;

/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/**
 * @author Oliver
 *
 */
public class CalcFibonacci
{

	/**
	 * Creates a new <code>CalcFibonacci</code> object.
	 */
	public CalcFibonacci()
	{
		super();
	}
	
	public static void main(String[] args)
	{
		int iterations = 10;
		int fiboNumber = 50000;
		if(args != null && args.length >= 1)
		{
			iterations = Integer.valueOf(args[0]).intValue();
			if(args.length >= 2)
				fiboNumber = Integer.valueOf(args[1]).intValue();
		}
		new CalcFibonacci().calc(fiboNumber, iterations);
	}
	
	protected void calc(int nthFibonacciNumber, int iterations)
	{		
		long total = 0;
		for (int i = 0; i < iterations; i++)
		{
			// System.gc();
			long time = System.currentTimeMillis();
			calcPrim(nthFibonacciNumber);
			time = System.currentTimeMillis() - time;
			total += time;
			System.out.println("Calculated " + nthFibonacciNumber + " Fibonacci in " + time + " ms"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}
		System.out.println("average: " + total / (float) iterations); //$NON-NLS-1$
	}
	
	protected BigInteger calcPrim(int nthFibonacciNumber)
	{
		if (nthFibonacciNumber < 3)
			return BigInteger.ZERO;
		BigInteger a1 = BigInteger.ZERO;
		BigInteger a2 = BigInteger.ONE;
		BigInteger an = BigInteger.ONE;
		for (int i = 0; i < nthFibonacciNumber; i++)
		{
			an = a1.add(a2);
			a1 = a2;
			a2 = an;
		}
		return an;
	}

}
