/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.demo.fibonacci;

import java.util.ArrayList;
import java.util.List;

import org.almendra.commons.IHistorySummary;
import org.almendra.commons.util.logging.LoggerAccessor;
import org.almendra.commons.util.swing.*;
import org.almendra.janet.cas.scheduling.CommandHistorySummary;



public class HistoryTotalsCreator
{
	protected int summaryCounter = -1;
	protected List<IHistorySummary> summaries = new ArrayList<IHistorySummary>();
	protected int fibonacciSize = -1;
	protected LoggerAccessor logAccessor = null;
	protected long startTime = -1;

	/**
	 * Creates a new <code>HistoryTotalsCreator</code> object.
	 */
	public HistoryTotalsCreator(int summaryCounter, int fibonacciSize, LoggerAccessor logAccessor, long startTime)
	{
		super();
		this.summaryCounter = summaryCounter;
		this.logAccessor = logAccessor;
		this.fibonacciSize = fibonacciSize;
		this.startTime = startTime;
	}
	
	/**
	 * Creates a new <code>HistoryTotalsCreator</code> object.
	 */
	protected HistoryTotalsCreator()
	{
		super();
	}
	
	public synchronized void addHistoryItems(List historyItems, JTextPaneHolder textPaneHolder)
	{
		CommandHistorySummary summary = new CommandHistorySummary();
		summary.create(historyItems);
		summaries.add(summary);
		
		if(summaries.size() == summaryCounter)
			createTotals(textPaneHolder);		
	}

	/**
	 * 
	 */
	public void createTotals(JTextPaneHolder textPaneHolder)
	{
		long totalDuration = System.currentTimeMillis() - startTime;
		long duration = 0;
		CommandHistorySummary totals = new CommandHistorySummary(); 
		for (int i = 0; i < summaries.size(); i++)
		{
			CommandHistorySummary summary = (CommandHistorySummary) summaries.get(i);
			totals.add(summary);
			duration += summary.getTotalTime();
		}		
		
		StringBuffer buffer = new StringBuffer();
		buffer.append("\nTOTAL TIME SPENT CALCULATION FIBONACCI NUMBERS using " + summaries.size() + " agents: "); //$NON-NLS-1$ //$NON-NLS-2$
		buffer.append("\n" + duration + " ms = " + (duration / 1000) + " seconds " + (duration - 1000 * (duration / 1000)) + " ms ");   //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		buffer.append("\n\nTOTAL TIME SPENT TILL ALL RESULTS RECEIVED using " + summaries.size() + " agents: "); //$NON-NLS-1$ //$NON-NLS-2$
		buffer.append("\n" + totalDuration + " ms = " + (totalDuration / 1000) + " seconds " + (totalDuration - 1000 * (totalDuration / 1000)) + " ms ");   //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$		
		System.out.println(buffer);
		
		buffer = new StringBuffer();
		buffer.append("\nTIME SPENT CALCULATION FIBONACCI NUMBERS using " + summaries.size() + " agents: "); //$NON-NLS-1$ //$NON-NLS-2$
		buffer.append("\n- systemStayTime=" + totals.getSystemStayTime()); //$NON-NLS-1$
		buffer.append(" / travelTime=" + totals.getTravelTime()); //$NON-NLS-1$
		buffer.append(" / executionTime=" + totals.getExecutionTime()); //$NON-NLS-1$
		buffer.append(" / totalTime=" + totals.getTotalTime()); //$NON-NLS-1$
		buffer.append("\n"); //$NON-NLS-1$
		
		if(textPaneHolder != null)
			textPaneHolder.addText(buffer.toString());
		
		logAccessor.freeText(buffer.toString());
	}	

}
