/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.logger;

import java.util.List;

import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.logging.ILogListener;
import org.almendra.janet.cas.agents.AgentProxy;
import org.almendra.janet.cas.logger.commands.DisplayLogEntryCommand;
import org.almendra.janet.cas.node.LogEntry;


/**
 * @author Oliver
 *
 */
public class LogListener implements ILogListener<LogEntry>
{
	protected LogCollector<LogEntry> collector = null;
	
	/**
	 * Creates a new <code>LogListener</code> object.
	 */
	protected LogListener()
	{
		super();
	}
	
	/**
	 * Creates a new <code>LogListener</code> object.
	 */
	public LogListener(LogCollector<LogEntry> collector)
	{
		super();
		this.collector = collector;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.util.logging.ILogListener#log(org.almendra.commons.commons.util.logging.LogEntry)
	 */
	public void log(LogEntry entry)
	{
		if(preventEndlessLoop(entry))
			return;
		collector.log(entry);
	}

	/**
	 * @param entry
	 * @return
	 */
	protected boolean preventEndlessLoop(LogEntry entry)
	{
		if(entry.getArgument() == null)
			return false;
		if(entry.getEvent() == null)
			return false;
		if(!entry.getEvent().equals(AgentProxy.LOG_EVENT_SENT_COMMAND))
			return false;
		return StringUtility.contains(entry.getArgument(), DisplayLogEntryCommand.ShortName);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.util.logging.ILogListener#log(java.util.List)
	 */
	public void log(List<LogEntry> entries)
	{
		collector.log(entries);		
	}

}
