/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.logger.interpreters;

import java.util.List;

import org.almendra.commons.events.Event;
import org.almendra.commons.events.IEventHandler;
import org.almendra.commons.util.collection.StringList;
import org.almendra.commons.util.logging.AddLogListenerParams;
import org.almendra.commons.util.logging.ILogListener;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.events.EventHandlerEnvelope;
import org.almendra.janet.cas.events.HandlerList;
import org.almendra.janet.cas.handlers.NodeDeregisteredParams;
import org.almendra.janet.cas.logger.LogCollector;
import org.almendra.janet.cas.logger.LogListener;
import org.almendra.janet.cas.logger.commands.InstallLoggerCommand;
import org.almendra.janet.cas.logger.handlers.NodeDeregisteredHandler;
import org.almendra.janet.cas.node.IllegalAgentAccessException;
import org.almendra.janet.cas.node.NoSuchAgentException;
import org.almendra.janet.cas.scheduling.CommandAccessor;
import org.almendra.janet.cas.scheduling.IInterpreter;


/**
 * @author Oliver
 *
 */
public class InstallLoggerInterpreter extends LoggerInterpreter implements IInterpreter
{

	/**
	 * Creates a new <code>InstallLoggerInterpreter</code> object.
	 */
	public InstallLoggerInterpreter()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.schedulers.IInterpreter#execute(org.almendra.commons.janet.cas.logger.schedulers.CommandAccessor)
	 */
	public void execute(CommandAccessor cmdAccessor)
	{
		if(cmdAccessor.getCommand() instanceof InstallLoggerCommand)
		{
			try
			{
				InstallLoggerCommand cmd = (InstallLoggerCommand) cmdAccessor.getCommand();
				ILogListener logListener = installLoggerListener(cmdAccessor, cmd);
				if(logListener != null)
					registerForEvents(cmdAccessor, logListener);
			}
			catch (Exception e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				cmdAccessor.log(e);
			}
		}
	}

	/**
	 * @param cmdAccessor
	 */
	protected void registerForEvents(CommandAccessor cmdAccessor, ILogListener logListener)
	{
		IEventHandler handler = new NodeDeregisteredHandler(cmdAccessor.getNodeAccessor(), cmdAccessor.getLogger(), logListener);
		EventHandlerEnvelope env = new EventHandlerEnvelope(handler, cmdAccessor.getAgentPath());
		HandlerList handlerList = new HandlerList(env);
		cmdAccessor.getNodeEventRegistry().register(NodeDeregisteredParams.EVENT_NAME, handlerList, cmdAccessor.getApplication());
	}

	protected ILogListener installLoggerListener(CommandAccessor cmdAccessor, InstallLoggerCommand cmd) 
	throws NoSuchAgentException, IllegalAgentAccessException
	{
		String listenerKey = cmd.getAgentPathToLoggerServer().getQualifiedPathName();
		if(cmdAccessor.getApplicationObjectSpace().contains(listenerKey))
			return null;
		String collectorKey = getCollectorKey(listenerKey);
		IAgentProxy consumerAgent = cmdAccessor.getAgent(cmd.getAgentPathToLoggerServer());
		LogCollector logCollector = new LogCollector(consumerAgent);
		ILogListener logListener = new LogListener(logCollector);
		cmdAccessor.getApplicationObjectSpace().put(collectorKey, logCollector);
		cmdAccessor.getApplicationObjectSpace().put(listenerKey, logListener);
		AddLogListenerParams params = new AddLogListenerParams(logListener, cmdAccessor.getLogger());
		Event event = new Event(AddLogListenerParams.EVENT_NAME, params);
		cmdAccessor.getNodeEventRegistry().occured(event);
		return logListener;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.scheduling.IInterpreter#commandNames()
	 */
	public List<String> commandNames()
	{
		return new StringList(InstallLoggerCommand.QualifiedName);
	}

}
