/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.monitor;

import java.util.HashMap;
import java.util.Map;

import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.events.HandlerList;
import org.almendra.janet.cas.monitor.commands.DisplayAttributeCommand;
import org.almendra.janet.cas.node.IMonitorProducerAnchor;
import org.almendra.janet.cas.node.NodeAnchor;
import org.almendra.janet.cas.scheduling.ICommand;
import org.almendra.janet.cas.scheduling.ICommandEnvelope;



public class MonitorProducerAnchor extends MonitorAnchor implements IMonitorProducerAnchor
{
	public static final String LABEL = "MONITOR_PRODUCER_ANCHOR"; //$NON-NLS-1$
	public static final String CAPABILITY_NAME = "MONITOR"; //$NON-NLS-1$
	public static final String AGENT_NAME = "PRODUCER"; //$NON-NLS-1$
	
	protected Map<String, IAgentProxy> consumerAgentPaths = new HashMap<String, IAgentProxy>();
	protected Map<String, HandlerList> shutdownHandlers = new HashMap<String, HandlerList>();
	
	/**
	 * Creates a new <code>MonitorProducerAnchor</code> object.
	 */
	public MonitorProducerAnchor()
	{
		super();
	}
	
	/**
	 * @return Returns the consumerAgentPaths.
	 */
	public Map<String, IAgentProxy> getConsumerAgentPaths()
	{
		return consumerAgentPaths;
	}
	
	/**
	 * @return Returns the shutdownHandlers.
	 */
	public Map<String, HandlerList> getShutdownHandlers()
	{
		return shutdownHandlers;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.node.IMonitorProducerAnchor#getStartedCommandNotification(org.almendra.commons.janet.cas.scheduling.CommandEnvelope)
	 */
	public ICommand getStartedCommandNotification(String nodeName, ICommandEnvelope env)
	{
		DisplayAttributeCommand cmd = new DisplayAttributeCommand(nodeName);
		DisplayAttributeCommand.Entry entry = cmd.new Entry();
		
		if(env.isForeign())
			entry.attribute = NodeAnchor.FOREIGN_EXECUTING_COMMANDS_COUNT;
		else
			entry.attribute = NodeAnchor.LOCAL_EXECUTING_COMMANDS_COUNT;
		
		entry.operation = DisplayAttributeCommand.OperationIncrement;
		cmd.add(entry);
		
		entry = cmd.new Entry();		
		if(env.isForeign())
			entry.attribute = NodeAnchor.FOREIGN_WAITING_COMMANDS_COUNT;
		else
			entry.attribute = NodeAnchor.LOCAL_WAITING_COMMANDS_COUNT;
		
		entry.operation = DisplayAttributeCommand.OperationDecrement;
		cmd.add(entry);
		return cmd;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.node.IMonitorProducerAnchor#getFinishedCommandNotification(org.almendra.commons.janet.cas.scheduling.ICommandEnvelope)
	 */
	public ICommand getFinishedCommandNotification(String nodeName, ICommandEnvelope env)
	{
		DisplayAttributeCommand cmd = new DisplayAttributeCommand(nodeName);
		DisplayAttributeCommand.Entry entry = cmd.new Entry();
		
		if(env.isForeign())
			entry.attribute = NodeAnchor.FOREIGN_EXECUTING_COMMANDS_COUNT;
		else
			entry.attribute = NodeAnchor.LOCAL_EXECUTING_COMMANDS_COUNT;
		
		entry.operation = DisplayAttributeCommand.OperationDecrement;
		cmd.add(entry);	
		return cmd;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.node.IMonitorProducerAnchor#getReceivedCommandNotification(org.almendra.commons.janet.cas.scheduling.ICommandEnvelope)
	 */
	public ICommand getReceivedCommandAndWaitingNotification(String nodeName, ICommandEnvelope env)
	{
		DisplayAttributeCommand cmd = new DisplayAttributeCommand(nodeName);
		DisplayAttributeCommand.Entry entry = cmd.new Entry();
		
		if(env.isForeign())
			entry.attribute = NodeAnchor.FOREIGN_COMMANDS_COUNT;
		else
			entry.attribute = NodeAnchor.LOCAL_COMMANDS_COUNT;
		
		entry.operation = DisplayAttributeCommand.OperationIncrement;
		cmd.add(entry);
		
		entry = cmd.new Entry();		
		if(env.isForeign())
			entry.attribute = NodeAnchor.FOREIGN_WAITING_COMMANDS_COUNT;
		else
			entry.attribute = NodeAnchor.LOCAL_WAITING_COMMANDS_COUNT;
		
		entry.operation = DisplayAttributeCommand.OperationIncrement;
		cmd.add(entry);
		return cmd;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.node.IMonitorProducerAnchor#getRemovedWaitingCommandNotification(java.lang.String, org.almendra.commons.janet.cas.scheduling.ICommandEnvelope)
	 */
	public ICommand getRemovedWaitingCommandNotification(String nodeName, ICommandEnvelope env)
	{
		DisplayAttributeCommand cmd = new DisplayAttributeCommand(nodeName);
		DisplayAttributeCommand.Entry entry = cmd.new Entry();		
		entry.attribute = NodeAnchor.EVICTED_WAITING_COMMANDS_COUNT;
		
		entry.operation = DisplayAttributeCommand.OperationIncrement;
		cmd.add(entry);

		return cmd;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.node.IMonitorProducerAnchor#getEvictedWaitingCommandNotification(java.lang.String, org.almendra.commons.janet.cas.scheduling.ICommandEnvelope)
	 */
	public ICommand getEvictedWaitingCommandNotification(String nodeName, ICommandEnvelope env)
	{
		DisplayAttributeCommand cmd = new DisplayAttributeCommand(nodeName);
		DisplayAttributeCommand.Entry entry = cmd.new Entry();
		
		entry.attribute = NodeAnchor.EVICTED_WAITING_COMMANDS_COUNT;		
		entry.operation = DisplayAttributeCommand.OperationIncrement;
		cmd.add(entry);
		
		entry = cmd.new Entry();		
		if(env.isForeign())
			entry.attribute = NodeAnchor.FOREIGN_WAITING_COMMANDS_COUNT;
		else
			entry.attribute = NodeAnchor.LOCAL_WAITING_COMMANDS_COUNT;		
		entry.operation = DisplayAttributeCommand.OperationDecrement;
		cmd.add(entry);
		
		return cmd;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.node.IMonitorProducerAnchor#getEvictedExecutingCommandNotification(java.lang.String, org.almendra.commons.janet.cas.scheduling.ICommandEnvelope)
	 */
	public ICommand getEvictedExecutingCommandNotification(String nodeName, ICommandEnvelope env)
	{
		DisplayAttributeCommand cmd = new DisplayAttributeCommand(nodeName);
		DisplayAttributeCommand.Entry entry = cmd.new Entry();
		
		entry.attribute = NodeAnchor.EVICTED_EXECUTING_COMMANDS_COUNT;		
		entry.operation = DisplayAttributeCommand.OperationIncrement;
		cmd.add(entry);
		
		return cmd;
	}

	public ICommand getReceivedCommandNotification(String nodeName, ICommandEnvelope env)
	{
		DisplayAttributeCommand cmd = new DisplayAttributeCommand(nodeName);
		DisplayAttributeCommand.Entry entry = cmd.new Entry();
		
		if(env.isForeign())
			entry.attribute = NodeAnchor.FOREIGN_COMMANDS_COUNT;
		else
			entry.attribute = NodeAnchor.LOCAL_COMMANDS_COUNT;
		
		entry.operation = DisplayAttributeCommand.OperationIncrement;
		cmd.add(entry);
		
		return cmd;
	}

}
