/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.monitor.handlers;

import java.rmi.RemoteException;

import org.almendra.commons.events.Event;
import org.almendra.commons.events.IEventHandler;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.monitor.commands.UnInstallProducerCommand;
import org.almendra.janet.cas.monitor.commands.UnsetMaskCommand;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.InvalidCommandException;
import org.almendra.janet.cas.scheduling.ICommand;



public class NodeShutdownHandler implements IEventHandler
{
	protected String nodeName = null;
	protected AgentPath consumerAgentPath = null;
	protected IAgentProxy localAgent = null;
	protected IAgentProxy consumerAgent = null;
	
	/**
	 * Creates a new <code>NodeShutdownHandler</code> object.
	 */
	protected NodeShutdownHandler()
	{
		super();
	}
	
	/**
	 * Creates a new <code>NodeShutdownHandler</code> object.
	 */
	public NodeShutdownHandler(String nodeName, IAgentProxy localAgent, IAgentProxy consumerAgent, AgentPath consumerAgentPath)
	{
		super();
		this.nodeName = nodeName;
		this.localAgent = localAgent;
		this.consumerAgent = consumerAgent;
		this.consumerAgentPath = consumerAgentPath;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.IEventHandler#occured(org.almendra.commons.commons.events.Event)
	 */
	public void occured(Event event)
	{
		try
		{
			ICommand cmd = new UnsetMaskCommand(nodeName);
			consumerAgent.accept(cmd);
			cmd = new UnInstallProducerCommand(consumerAgentPath);
			localAgent.accept(cmd);
		}
		catch (RemoteException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		catch (InvalidCommandException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
}
