/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
package org.almendra.janet.cas.events;

import java.io.Serializable;

import org.almendra.commons.events.Event;
import org.almendra.commons.events.IEventHandler;
import org.almendra.janet.cas.agents.AgentProxy;
import org.almendra.janet.cas.commands.ExecuteEventHandlerCommand;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.ApplicationAccessor;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.node.interpreters.ExecuteEventHandlerInterpreter;
import org.almendra.janet.cas.scheduling.CommandEnvelope;
import org.almendra.janet.cas.scheduling.ICommandEnvelope;



public class EventHandlerEnvelope extends AbstractEventHandlerEnvelope implements IEventHandler, Serializable
{
	private static final long serialVersionUID = 5299575790280173078L;
	
	protected Node node = null;
	protected transient ApplicationAccessor application = null;
	
	/**
	 * @return Returns the application.
	 */
	public ApplicationAccessor getApplication()
	{
		return application;
	}

	/**
	 * @param application The application to set.
	 */
	public void setApplication(ApplicationAccessor application)
	{
		this.application = application;
	}

	/**
	 * Creates a new <code>EventHandlerEnvelope</code> object.
	 */
	protected EventHandlerEnvelope()
	{
		super();
	}
	
	/**
	 * Creates a new <code>EventHandlerEnvelope</code> object.
	 */
	public EventHandlerEnvelope(IEventHandler handler, AgentPath agentPath)
	{
		super(handler, agentPath);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.commons.events.IEventHandler#occuredEvent(org.almendra.commons.commons.events.Event)
	 */
	public void occured(Event event)
	{		
		getAgentPath().checkValidity();
		ExecuteEventHandlerCommand cmd = new ExecuteEventHandlerCommand(handler, event);
		ICommandEnvelope env = new CommandEnvelope(getAgentPath(), cmd, node.getLogger(), node.getName());
		env.setPredefinedInterpreter(new ExecuteEventHandlerInterpreter());
		
		try
		{
			AgentProxy agent = application.getCrossAgent(getAgentPath());
			agent.accept(env);
		} 
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			node.getLogger().exception(e);
		}
	}

	/**
	 * @return Returns the node.
	 */
	public Node getNode()
	{
		return node;
	}

	/**
	 * @param node The node to set.
	 */
	public void setNode(Node node)
	{
		this.node = node;
	}

}
