/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node;

import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.List;

import org.almendra.commons.INamedObject;
import org.almendra.commons.util.AssertUtility;


/**
 * @author Oliver
 *
 */
public abstract class AbstractNodeAnchor  implements INamedObject
{
	public static final String LABEL = "NodeAnchor"; //$NON-NLS-1$
	
	protected Node node = null;
	protected NodeDescriptor nodeDescriptor = null; 
	protected String viewTitle = null;
	protected String label = null;
	
	/**
	 * @return Returns the viewTitle.
	 */
	public String getViewTitle()
	{
		return viewTitle;
	}

	/**
	 * @param viewTitle The viewTitle to set.
	 */
	public void setViewTitle(String viewTitle)
	{
		this.viewTitle = viewTitle;
	}

	protected AbstractNodeAnchor()
	{
		super();
	}
	
	protected AbstractNodeAnchor(Node node)
	{
		super();
		this.node = node;
	}

	/**
	 * @return Returns the node.
	 */
	public Node getNode()
	{
		return node;
	}

	protected String name = null;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.almendra.commons.INamedObject#getName()
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * @param nodeName
	 */
	public void setName(String nodeName)
	{
		name = nodeName;
		node.setName(nodeName);
	}

	/**
	 * @param list
	 */
	public void registerForeignApplication(NodeLocation location, List<AgentPath> agentPaths)
	{
	    for(AgentPath path : agentPaths)
		{	
			try
			{
				AbstractNodeImage nodeImage = getNodeImage(location, path.getNodeName());
				nodeImage.registerForeignApplicationAgent(path);
			}
			catch (Exception e)
			{
				System.out.println(e.getMessage());
				e.printStackTrace();
				node.getLogger().exception(e);
			}
		}
	}
	
	/**
	 * Deregister all applications of a foreign node by remove the entire foreign node image.
	 * Usually only done when node is shutting down.
	 * @param path
	 */
	public void deregisterForeignApplications(String foreignNodeName)
	{
		if(!name.equals(foreignNodeName))
			getClusterImage().deregisterAllApplications(foreignNodeName);
	}
	
	public abstract AbstractClusterImage getClusterImage();
	
	public AbstractNodeImage deregisterNode(String nodeName)
	{
		return getClusterImage().removeNodeImage(nodeName);
	}

	/**
	 * @param string
	 * @param string2
	 */
	public void deregisterForeignApplication(String applicationName, String nodeName)
	{
		AbstractNodeImage nodeImage = getNodeImage(nodeName);
		nodeImage.deregisterForeignApplication(applicationName);
	}

	protected AbstractNodeImage getNodeImage(String nodeName)
	{
		return getClusterImage().getNodeImage(nodeName);
	}
	
	protected AbstractNodeImage getNodeImage(NodeLocation location, String nodeName) 
	throws AccessException, RemoteException, NotBoundException
	{
		AssertUtility.assertTrue(location.getNodeName().equals(nodeName));
		return getClusterImage().getNodeImage(location, nodeName);
	}

	/**
	 * @param applicationName
	 * @param nodeName
	 * @return
	 */
	public boolean isForeignApplicationRegistered(String applicationName, String nodeName)
	{
		AbstractNodeImage nodeImage = getNodeImage(nodeName);
		return nodeImage.isForeignApplicationRegistered(applicationName);
	}

	/**
	 * @param path
	 * @return
	 */
	public boolean isForeignCapabilityRegistered(CapabilityPath path)
	{
		AbstractNodeImage nodeImage = getNodeImage(path.getNodeName());
		if(nodeImage == null)
			return false;
		return nodeImage.isForeignCapabilityRegistered(path.getApplicationName(), path.getCapabilityName());
	}
	
	/**
	 * @return Returns the centralAgentDispatcher.
	 */
	public abstract IAgentDispatcher getCentralDispatcher();

	/**
	 * @param node
	 */
	public abstract boolean disconnectRequest(Node node);
	
	public boolean mayDisconnect()
	{
		return true;
	}

	/**
	 * @return Returns the nodeDescriptor.
	 */
	public NodeDescriptor getNodeDescriptor()
	{
		return nodeDescriptor;
	}

	/**
	 * @param nodeDescriptor The nodeDescriptor to set.
	 */
	public void setNodeDescriptor(NodeDescriptor nodeDescriptor)
	{
		this.nodeDescriptor = nodeDescriptor;
	}

	/**
	 * @return Returns the label.
	 */
	public String getLabel()
	{
		return label;
	}
	/**
	 * @param label The label to set.
	 */
	public void setLabel(String label)
	{
		this.label = label;
	}
}
