/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node;

import org.almendra.commons.util.StringUtility;
import org.almendra.commons.util.xml.XMLParseException;
import org.jdom.Element;


/**
 * @author Oliver
 *
 */
public class AgentDescriptor extends Descriptor
{
	public static final String XMLExecuteWhenAppStartedTag = "executeWhenStarted"; //$NON-NLS-1$
	public static final String XMLAgentNameTag = "name"; //$NON-NLS-1$
	
	protected String name = null;
	protected String executeWhenAppStarted = null;
	
	/**
	 * Creates a new <code>AgentDescriptor</code> object.
	 */
	public AgentDescriptor()
	{
		super();
	}
	
	/**
	 * Creates a new <code>AgentDescriptor</code> object.
	 */
	public AgentDescriptor(String name)
	{
		super();
		this.name = name;
	}

	/**
	 * @return Returns the name.
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * @param name The name to set.
	 */
	public void setName(String name)
	{
		this.name = name;
	}
	
	public static AgentDescriptor create(Element node) throws XMLParseException
	{
		AgentDescriptor desc = new AgentDescriptor();
		String value = node.getAttributeValue(XMLAgentNameTag);
		if (!StringUtility.isValid(value))
			throw new XMLParseException("no valid agent name specified!"); //$NON-NLS-1$
		desc.setName(value);
		// may be null
		desc.setExecuteWhenAppStarted(node.getAttributeValue(XMLExecuteWhenAppStartedTag));
		return desc;
	}

	/**
	 * @return Returns the executeWhenAppStarted.
	 */
	public String getExecuteWhenAppStarted()
	{
		return executeWhenAppStarted;
	}

	/**
	 * @param executeWhenAppStarted The executeWhenAppStarted to set.
	 */
	public void setExecuteWhenAppStarted(String executeWhenAppStarted)
	{
		this.executeWhenAppStarted = executeWhenAppStarted;
	}

}
