/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.node;

import org.almendra.commons.util.xml.XMLParseException;
import org.almendra.janet.cas.agents.*;
import org.jdom.Element;


public class SystemApplicationDescriptor extends ApplicationDescriptor
{
	public static final String XMLNodeTag = "systemApplication"; //$NON-NLS-1$
	public static final String NAME = "SYSTEM"; //$NON-NLS-1$
	public static final String AGENT_NAME = "SYSTEM"; //$NON-NLS-1$

	/**
	 * Creates a new <code>SystemApplicationDescriptor</code> object.
	 *  
	 */
	public SystemApplicationDescriptor()
	{
		super();
	}

	public static SystemApplicationDescriptor createSysAppDesc(Element element) throws XMLParseException
	{
		if (element == null)
			throw new XMLParseException("no system application specified!"); //$NON-NLS-1$
		SystemApplicationDescriptor desc = new SystemApplicationDescriptor();
		desc.fill(element);
		return desc;
	}

	protected void checkValidity() throws XMLParseException
	{
		CapabilityDescriptor coreCap = null;	
		for(CapabilityDescriptor capDesc : getCapabilityDescriptors())
		{
			if (capDesc.getName().equals(SystemApplication.CORE_CAPABILITY_NAME))
			{
				coreCap = capDesc;
				break;
			}
		}
		if (coreCap == null)
			throw new XMLParseException("no CORE capability defined for system application!"); //$NON-NLS-1$
		if(coreCap.getAgentNames().size() == 0)
			throw new XMLParseException("no agent specified for CORE capability of system application!"); //$NON-NLS-1$
		if(coreCap.getAgentNames().size() > 1)
			throw new XMLParseException("no user defined agents allowed for CORE capability of system application!"); //$NON-NLS-1$
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.ApplicationDescriptor#fillCapabilities(org.jdom.Element)
	 */
	protected void fillCapabilities(Element element) throws XMLParseException
	{
		super.fillCapabilities(element);
		for(CapabilityDescriptor capDesc : getCapabilityDescriptors())
		{
			if(capDesc.getName().equals(SystemApplication.CORE_CAPABILITY_NAME))
				capDesc.addAgentName(SystemAgent.NAME);
			else
				capDesc.checkHasAgents();
		}
		checkValidity();
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.ApplicationDescriptor#isMustHaveAgentsDefined()
	 */
	protected boolean isMustHaveAgentsDefined()
	{
		return false;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.logger.node.ApplicationDescriptor#setName(org.jdom.Element)
	 */
	protected void setName(Element element)
	{
		setName(NAME);
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.janet.cas.node.ApplicationDescriptor#createEventDescriptor(org.jdom.Element)
	 */
	protected EventDescriptor createEventDescriptor(Element eventNode) throws XMLParseException
	{
		EventDescriptor desc = EventDescriptor.createNoAgent(eventNode);
		desc.setAgentPathString(SystemAgent.CAPABILITY_NAME + "." + SystemAgent.NAME); //$NON-NLS-1$
		return desc;
	}

}
