/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node.interpreters;

import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import org.almendra.commons.events.Event;
import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.inet.RegistryLocation;
import org.almendra.janet.cas.handlers.NodeRegisteredParams;
import org.almendra.janet.cas.node.IAgentDispatcher;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.node.NodeAnchor;
import org.almendra.janet.cas.node.NodeLocation;
import org.almendra.janet.cas.scheduling.CommandAccessor;


/**
 * Abstract trait class for all trait classes that deal with registering nodes.  
 * For a further introduction into traits see http://www.iam.unibe.ch/~scg/Research/Traits/
 * 
 * @author Oliver
 *
 */
public abstract class AbstractRegisterNodeTrait
{

	/**
	 * Creates a new <code>AbstractRegisterNodeTrait</code> object.
	 */
	public AbstractRegisterNodeTrait()
	{
		super();
	}

	public abstract RegistryLocation getRegistryLocation(CommandAccessor cmdAccessor);

	public void registerForeignNode(NodeLocation location, CommandAccessor cmdAccessor)
	{
		// register foreign node in my node image
		IAgentDispatcher agentDispatcher = null;
		try
		{
			agentDispatcher = getAgentDispatcher(location, cmdAccessor.getLogger());
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.log(e);
			return;
		}
		
		AssertUtility.assertTrue(agentDispatcher != null);
		NodeAnchor anchor = cmdAccessor.getNodeAnchorIfPrivilege();
		anchor.addForeignNode(agentDispatcher, location);
	}

	/**
	 * @param location
	 * @return
	 */
	public abstract IAgentDispatcher getAgentDispatcher(NodeLocation location, Logger logger)
	throws AccessException, RemoteException, NotBoundException;

	public void sendEventNodeAboutToRegister(CommandAccessor cmdAccessor, String nodeName)
	{
		NodeRegisteredParams params = new NodeRegisteredParams(nodeName);
		Event event = new Event(NodeRegisteredParams.EVENT_NAME, params);
		cmdAccessor.getNodeEventRegistry().occured(event);
	}
	
}
