/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code or
 * in binary form must reproduce the above copyright notice, and the disclaimer
 * that follows.
 * 
 * In addition, you are required that you include in the end-user documentation
 * provided with the redistribution and/or in the software itself an
 * acknowledgement equivalent to the following: "This product is based on the
 * Janet software developed by the JANET Project
 * (http://www.almendra.org/janet/)."
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE JDOM
 * AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package org.almendra.janet.cas.node.interpreters;

import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.Registry;

import org.almendra.commons.events.Event;
import org.almendra.commons.util.inet.RMIUtility;
import org.almendra.commons.util.inet.RegistryLocation;
import org.almendra.janet.cas.commands.DeregisterNodeCommand;
import org.almendra.janet.cas.commands.DeregisterNodeFinalCommand;
import org.almendra.janet.cas.handlers.NodeDeregisteredParams;
import org.almendra.janet.cas.node.AbstractNodeAnchor;
import org.almendra.janet.cas.scheduling.CommandAccessor;


/**
 * @author Oliver
 *  
 */
public class DeregisterNodeTrait
{

	/**
	 * Creates a new <code>DeregisterNodeTrait</code> object.
	 */
	public DeregisterNodeTrait()
	{
		super();
	}

	public void deregisterNode(CommandAccessor cmdAccessor)
	{
		DeregisterNodeCommand command = (DeregisterNodeCommand) cmdAccessor.getCommand();
		AbstractNodeAnchor anchor = cmdAccessor.getAbstractNodeAnchorIfPrivilege();
		String nodeName = command.getNodeName();

		sendEventNodeAboutToDeregister(cmdAccessor, nodeName);
		anchor.deregisterForeignApplications(nodeName);
		deregisterNodeFinally(cmdAccessor, nodeName);
	}

	public void deregisterCentral(CommandAccessor cmdAccessor)
	{
		DeregisterNodeCommand command = (DeregisterNodeCommand) cmdAccessor.getCommand();
		String nodeName = command.getNodeName();
		sendEventNodeAboutToDeregister(cmdAccessor, nodeName);
	}
	
	protected void sendEventNodeAboutToDeregister(CommandAccessor cmdAccessor, String nodeName)
	{
		NodeDeregisteredParams params = new NodeDeregisteredParams(nodeName);
		Event event = new Event(NodeDeregisteredParams.EVENT_NAME, params);
		cmdAccessor.getNodeEventRegistry().occured(event);
	}

	/**
	 * Add a command to the local system agent's schedulerAnchor to remove the
	 * reference of the node to be deregistered. The command will be executed
	 * after the one it was spawned from has finished execution. This prevents
	 * the reference to the node to be removed before the acks to the system
	 * agent of the node to be deregistered have been sent away.
	 * 
	 * @param cmdAccessor
	 */
	protected void deregisterNodeFinally(CommandAccessor cmdAccessor, String nodeName)
	{
		try
		{
			cmdAccessor.getSystemAgent().accept(new DeregisterNodeFinalCommand(nodeName));
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.log(e);
		}
	}
	
	public void unbind(RegistryLocation registryLocation, String rmiNodeName) 
	throws AccessException, RemoteException, NotBoundException
	{
		Registry registry = RMIUtility.getRegistry(registryLocation);
		registry.unbind(rmiNodeName);
	}
}
