/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.node.interpreters;

import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.Registry;

import org.almendra.commons.util.exception.InsufficientPrivilegeException;
import org.almendra.commons.util.inet.RMIUtility;
import org.almendra.commons.util.inet.RegistryLocation;
import org.almendra.janet.cas.node.IAgentDispatcher;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.node.NodeLocation;
import org.almendra.janet.cas.scheduling.CommandAccessor;


/**
 * Trait class to define shared behavior of org.almendra.commons.janet.cas.logger.commands.RegisterNodeInterpreter, 
 * org.almendra.commons.janet.cas.logger.commands.central.RegisterNodeInterpreter and of org.almendra.commons.janet.cas.logger.commands.RegisteredNodeInterpreter.
 *  
 * For a further introduction into traits see http://www.iam.unibe.ch/~scg/Research/Traits/
 * 
 * @author Oliver
 *
 */
public class RegisterNodeTrait extends AbstractRegisterNodeTrait
{

	/**
	 * Creates a new <code>RegisterNodeTrait</code> object.
	 */
	public RegisterNodeTrait()
	{
		super();
	}
	
	public IAgentDispatcher getAgentDispatcher(NodeLocation location, Logger logger) 
	throws AccessException, RemoteException, NotBoundException
	{
		Registry registry = RMIUtility.getRegistry(location.getRegistryLocation());
		return (IAgentDispatcher) registry.lookup(Node.RMI_LABEL_PREFIX + location.getNodeName());
	}
	
	public RegistryLocation getRegistryLocation(CommandAccessor cmdAccessor)
	{
		try
		{
			return cmdAccessor.getSystemApplication().getCentralRegistryLocation();
		}
		catch (InsufficientPrivilegeException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			cmdAccessor.log(e);
		}
		return null;
	}

}
