/*
 * Created on 18.09.2003
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet/)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.schedulers;

import org.almendra.commons.util.AssertUtility;
import org.almendra.commons.util.StringUtility;
import org.almendra.janet.cas.node.AbstractApplication;
import org.almendra.janet.cas.node.Capability;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.scheduling.*;


public abstract class PriorityCommandScheduler<CommandEnvelopeType extends ICommandEnvelope> extends CommandScheduler<CommandEnvelopeType>
{
	public PriorityCommandScheduler()
	{
		super();
	}

	/**
	 * @param priority
	 * @param verbose
	 */
	public PriorityCommandScheduler(int priority, boolean verbose)
	{
		super(priority, verbose);
	}

	/**
	 * @param logger
	 */
	public PriorityCommandScheduler(Logger logger)
	{
		super(logger);
	}

	/**
	 * @param logger
	 * @param priority
	 */
	public PriorityCommandScheduler(Logger logger, int priority)
	{
		super(logger, priority);
	}

	/**
	 * @param logger
	 * @param priority
	 * @param verbose
	 */
	public PriorityCommandScheduler(
		Logger logger,
		int priority,
		boolean verbose)
	{
		super(logger, priority, verbose);
	}

	public PriorityCommandScheduler(int numPriorities)
	{
		super(numPriorities);
	}

	protected void init()
	{
		super.init();
		queue = new PriorityQueue<CommandEnvelopeType>(getDefaultNumberOfPriorities());
	}

	protected int getDefaultNumberOfPriorities()
	{
		return 5;
	}

	/* (non-Javadoc)
	 * @see org.almendra.commons.cas.node.scheduling.CommandScheduler#getInterpreter(org.almendra.commons.cas.node.ICommandEnvelope)
	 */
	public IInterpreter getInterpreter(ICommandEnvelope commandEnv)
	{
		IInterpreter interpreter = commandEnv.getPredefinedInterpreter();
		if(interpreter != null)
			return interpreter;
		
		String qualifiedName = commandEnv.getCommand().getQualifiedName();
		AssertUtility.assertTrue(StringUtility.isValid(qualifiedName));
		AbstractApplication app = getApplication(commandEnv.getApplicationName());
		if(app == null)
		{
			System.out.println("no application found named " + StringUtility.print(commandEnv.getApplicationName())); //$NON-NLS-1$
			return null;
		}
		
		Capability cap = app.getCapability(commandEnv.getCapabilityName());
		if(cap == null)
		{
			// TODO - add log message here
			System.out.println("no " + StringUtility.print(commandEnv.getCapabilityName()) + " capability found for application " + StringUtility.print(commandEnv.getApplicationName())); //$NON-NLS-1$ //$NON-NLS-2$
			return null;
		}
				
		return cap.getInterpreter(qualifiedName);
	}

	protected AbstractApplication getApplication(String applicationName)
	{
		return node.getApplication(applicationName);
	}

}
