/*
 * 
 * Copyright (C) 2004, 2005 Oliver Plohmann
 * All rights reserved.
 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source    
 * code or in binary form must reproduce the above copyright notice, 
 * and the disclaimer that follows.

 * In addition, you are required that you include in the  end-user 
 * documentation provided with the redistribution and/or in the 
 * software itself an acknowledgement equivalent to the following:
 * "This product is based on the Janet software developed by the 
 * JANET Project (http://www.almendra.org/janet)."

 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE JANET AUTHORS OR THE PROJECT
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package org.almendra.janet.cas.scheduling;

import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;
import java.util.NoSuchElementException;

import org.almendra.commons.Result;
import org.almendra.commons.threads.IExecutable;
import org.almendra.commons.util.exception.InsufficientPrivilegeException;
import org.almendra.commons.util.exception.InvalidMessageException;
import org.almendra.commons.util.globalId.UniqueId;
import org.almendra.janet.cas.agents.AgentProxy;
import org.almendra.janet.cas.agents.IAgent;
import org.almendra.janet.cas.agents.IAgentProxy;
import org.almendra.janet.cas.events.ILocalClusterEventRegistry;
import org.almendra.janet.cas.events.ILocalEventRegistry;
import org.almendra.janet.cas.node.AgentPath;
import org.almendra.janet.cas.node.IllegalAgentAccessException;
import org.almendra.janet.cas.node.LocalAgentPath;
import org.almendra.janet.cas.node.Logger;
import org.almendra.janet.cas.node.NoSuchAgentException;
import org.almendra.janet.cas.node.Node;
import org.almendra.janet.cas.node.NodeAccessor;
import org.almendra.janet.cas.node.NodeName;
import org.almendra.janet.cas.schedulers.CommandScheduler;
import org.almendra.janet.cas.spaces.IObjectSpace;

/**
 * @author Oliver Plohmann
 * @version 1.0, 23.03.2005
 * @since   1.0
 * Created on 23.03.2005
 */

public interface ICommandEnvelope extends IExecutable, ICommandHolder, Serializable, Cloneable, IPrioritizedObject
{
	public static final int COMMAND_TYPE_BALANCING = 1;
	public static final int COMMAND_TYPE_SHARING = 2;

	/**
     * @return
     */
    String toPrintString();
    
    /**
     * @return
     */
    IAgent getCurrentAgent();
    
    /**
     * @return
     */
    boolean isEventHandlerCommand();
    
    /**
     * @param scheduler
     */
    void setCurrentScheduler(CommandScheduler scheduler);
    
    /**
     * @return
     */
    IInterpreter getPredefinedInterpreter();
    
    /**
     * @return
     */
    String getApplicationName();

    /**
     * @return
     */
    String getCapabilityName();
    
    /**
     * @return
     */
    boolean isForeign();
	
	public Result invokeOnCommand(String methodName) 
	throws SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException;

	boolean isSuspended();

	void setCommand(ICommand executCommand);

	void setPredefinedInterpreter(IInterpreter predefinedInterpreter);
	
	public AgentProxy getCrossAgent(AgentPath path)
	throws NoSuchAgentException, IllegalAgentAccessException, InsufficientPrivilegeException;

	void setNewDestination(AgentPath selectedAgentPath, String nodeName);

	int getKind();

	AgentPath getOriginAgentPath();

	boolean testSetSuspended();

	AgentPath getAgentPath();

	AgentPath getDestinationAgent();

	boolean isWorkloadCommand();

	String getNodeName();

	String getQualifiedDestinationAgentName();

	String getOriginDestinationAgentName();

	UniqueId getGlobalId();

	void setKind(int kind);

	void aboutToBeSent();

	void setSendTime();

	void setResponse(Serializable replyObject) throws InvalidMessageException;

	void setCurrentAgent(IAgent agent);

	void setIgnore(boolean b);

	void setLogger(Logger logger);

	void setNode(Node node);

	boolean removeFromQueue();

	void log(Exception ex);

	String getLocalizedDestinationAgentName();

	Logger getLogger();

	boolean isSynchronousResponse();

	void arrivedAt(Node node);

	boolean isIgnore();

	String getQualifiedCapabilityName();

	Object getOriginNodeName();

	IObjectSpace getNodeObjectSpace();

	NodeName getNodeNameObject();

	ILocalEventRegistry getNodeEventRegistry();

	NodeAccessor getNodeAccessor();

	Node getNode() throws InsufficientPrivilegeException;

	CommandHistory getHistory();

	IAgentProxy getForeignAgent(String nodeName, String capabilityName) throws NoSuchAgentException, IllegalAgentAccessException;

	IAgentProxy getCurrentAgentProxy();

	ILocalClusterEventRegistry getClusterEventRegistry(String name) throws NoSuchElementException, RemoteException;

	String getCapabilityPath();

	IObjectSpace getApplicationObjectSpace();

	ILocalEventRegistry getApplicationEventRegistry();

	IAgentProxy getAllForeignAgents(String capabilityName) throws NoSuchAgentException, IllegalAgentAccessException;

	IAgentProxy getAgentProxy();

	LocalAgentPath getLocalAgentPath();
	
	Object clone() throws CloneNotSupportedException;

}
